import 'package:flutter/material.dart';

extension InkResponseClickableDetection on InkResponse {
  bool isCallbackClickExists() =>
    [onTap,
    onTapDown,
    onTapUp,
    onTapCancel,
    onDoubleTap,
    onLongPress].any((handler) => handler != null);
}

extension InkWellClickableDetection on InkWell {
  bool isCallbackClickExists() =>
    [onTap,
    onTapDown,
    onTapUp,
    onTapCancel,
    onDoubleTap,
    onLongPress].any((handler) => handler != null);
}

extension GestureDetectorClickableDetection on GestureDetector {
  bool isCallbackClickExists() => [
    onTapDown,
    onTapUp,
    onTap,
    onTapCancel,
    onDoubleTapDown,
    onDoubleTap,
    onDoubleTapCancel,
    onLongPressDown,
    onLongPressCancel,
    onLongPress,
    onLongPressStart,
    onLongPressMoveUpdate,
    onLongPressUp,
    onLongPressEnd,
  ].any((handler) => handler != null);
}

extension ButtonStyleButtonClickableDetection on ButtonStyleButton {
  bool isCallbackClickExists() => [onPressed, onLongPress].any((handler) => handler != null);
}

extension FloatingActionButtonClickableDetection on FloatingActionButton {
  bool isCallbackClickExists() => [onPressed].any((handler) => handler != null);
}

extension IconButtonClickableDetection on IconButton {
  bool isCallbackClickExists() => [onPressed].any((handler) => handler != null);
}

extension MaterialButtonClickableDetection on MaterialButton {
  bool isCallbackClickExists() => [onPressed, onLongPress].any((handler) => handler != null);
}

extension RadioListTileClickableDetection on RadioListTile {
  bool isCallbackClickExists() => [onChanged].any((handler) => handler != null);
}

extension PopupMenuButtonClickableDetection on PopupMenuButton {
  bool isCallbackClickExists() => [onOpened, onSelected, onCanceled].any((handler) => handler != null);
}

extension DropdownButtonClickableDetection on DropdownButton {
  bool isCallbackClickExists() => [onTap, onChanged].any((handler) => handler != null);
}

extension CheckboxClickableDetection on Checkbox {
  bool isCallbackClickExists() => [onChanged].any((handler) => handler != null);
}

extension ChipClickableDetection on Chip {
  bool isCallbackClickExists() => [onDeleted].any((handler) => handler != null);
}

extension ChoiceChipClickableDetection on ChoiceChip {
  bool isCallbackClickExists() => [onSelected].any((handler) => handler != null);
}

extension FilterChipClickableDetection on FilterChip {
  bool isCallbackClickExists() => [onSelected].any((handler) => handler != null);
}

extension InputChipClickableDetection on InputChip {
  bool isCallbackClickExists() => [onPressed, onSelected, onDeleted].any((handler) => handler != null);
}

extension ActionChipClickableDetection on ActionChip {
  bool isCallbackClickExists() => [onPressed].any((handler) => handler != null);
}

extension SwitchClickableDetection on Switch {
  bool isCallbackClickExists() => [onChanged].any((handler) => handler != null);
}

extension RadioClickableDetection on Radio {
  bool isCallbackClickExists() => [onChanged].any((handler) => handler != null);
}

extension ListTileClickableDetection on ListTile {
  bool isCallbackClickExists() => [onTap, onLongPress].any((handler) => handler != null);
}

extension TabBarClickableDetection on TabBar {
  bool isCallbackClickExists() => [onTap].any((handler) => handler != null);
}

extension BottomNavigationBarClickableDetection on BottomNavigationBar {
  bool isCallbackClickExists() => [onTap].any((handler) => handler != null);
}

extension SelectableTextClickableDetection on SelectableText {
  bool isCallbackClickExists() => [onTap, onSelectionChanged].any((handler) => handler != null);
}

extension TextFieldClickableDetection on TextField {
  bool isCallbackClickExists() => [onTap, onChanged, onEditingComplete, onSubmitted, onTapOutside].any((handler) => handler != null);
}