import 'dart:io';

import 'package:flutter/material.dart';
// !! we ignore using relative imports only for known issue with logger.dart static filds
// ignore: prefer_relative_imports
import 'package:pendo_sdk/src/logger.dart';
import '../constants/constants.dart';

extension ElementUtils on Element {

  // Used for Android platform to convert logical pixels to the actual when generating view hierarchy
  static double _devicePixelRatio = WidgetsBinding.instance.platformDispatcher.displays.first.devicePixelRatio; 

  static Rect? rectInScreenCoordinatesForAndroid(Rect? originalRect, double devicePixelRatio) {
    if (originalRect != null) {
      Rect devicePixelRect = Rect.fromLTRB(
          originalRect.left * devicePixelRatio, 
          originalRect.top * devicePixelRatio, 
          originalRect.right * devicePixelRatio, 
          originalRect.bottom * devicePixelRatio);
      return devicePixelRect;
    }
    return originalRect;
  }

  Rect? rectInScreenCoordinates() {
    final translation = renderObject?.getTransformTo(null).getTranslation();
    if (translation != null && renderObject?.paintBounds != null) {
      final offset = Offset(translation.x, translation.y);
      Rect? rect = renderObject!.paintBounds.shift(offset);
      if (Platform.isAndroid) {
        rect = rectInScreenCoordinatesForAndroid(rect, _devicePixelRatio);
      }
      return rect;
    }
    Logger.e('Element rect is null');
    assert(false, 'Element rect is null');
    return null;
  }

  bool isElementView() =>
      isElementLayoutImportantView() ||
      isElementUserResponsiveView() ||
      isElementText() || 
      widget is Icon || 
      widget is BackButtonIcon ||
      widget is Image || 
      widget is Ink ||
      widget is Column ||
      widget is Row ||
      widget is GridView ||
      widget is ListView ||
      widget is IndexedSemantics ||
      widget is BottomNavigationBar ||
      widget is PageView ||
      widget is TabBar ||
      widget is ListTile ||
      widget is TabPageSelectorIndicator ||
      widget is Tooltip ||
      widget is TabBarView ||
      widget is SliverFixedExtentList ||
      widget is GridPaper ||
      widget is Flow ||
      widget is DataTable ||
      widget.runtimeType.toString().contains(PROGRESS_INDICATOR_WIDGET);

  bool isElementText() =>
      widget is Text ||
      widget is SelectableText ||
      widget is RichText ||
      widget is TextField;

  bool isElementLayoutImportantView() => 
      widget is Navigator ||
      widget is LayoutId ||
      widget.runtimeType.toString().contains(THEATER_WIDGET) ||
      widget.runtimeType.toString().contains(OVERLAY_ENTRY_WIDGET);

  bool isElementUserResponsiveView() =>
      widget is Chip ||
      widget is ActionChip ||
      widget is ChoiceChip ||
      widget is FilterChip ||
      widget is InputChip ||
      widget is InkResponse || 
      widget is GestureDetector ||
      widget is ButtonStyleButton ||
      widget is FloatingActionButton ||
      widget is IconButton ||
      widget is MaterialButton ||
      widget is RadioListTile ||
      widget is PopupMenuButton ||
      widget is DropdownButton ||
      widget is Switch ||
      widget is Checkbox ||
      widget is Radio ||
      widget is Slider;
}
