import 'dart:convert';
import 'package:crypto/crypto.dart';
// ignore: prefer_relative_imports
import 'package:pendo_sdk/src/logger.dart';
import '../utils/common_utils.dart';

extension StringMatching on String {
  //MARK: this should be turned to TRUE  ONLY for debug purposes!!!
  static bool kDisableStringEncoding = false;
  String extractName() {
    //match anything in the single quotes
    final regex = RegExp(r"'([^']+)'");
    final matches = regex.allMatches(this);

    if (matches.isEmpty) {
      return '';
    }

    final unitedMatch = matches.map((match) => match.group(1) ?? '').join();
    return unitedMatch;
  }

  String extractReturnTypeFromBuilderDescription() {
    //The format we get  "xxx => match"
    final regex  = RegExp(r'=>\s*(.*)');
    final match = regex.firstMatch(this);
    if (match != null && match.groupCount >= 1) {
      return match.group(1)!;
    }
    return '';
  }

  String? validateText() {
    String? result = null;
  
    // we sometimes extract non valid unicode chars like: \UE093 which BE isn't processing well
    try {
      result = utf8.decode(this.codeUnits);
    } catch (e) {
      Logger.i('couldn\'t decode $this');
    }

    return result;
  }

  String? base64Str({int maxLength = 256 }) {
    if (kDisableStringEncoding || this.isEmpty) {
      return this;
    }
    
    String? validStr = this.validateText();
    if (CommonUtils.isNullOrEmptyString(validStr)) {
      return null;
    }
    
    final subStr = validStr!.length > maxLength ? validStr.substring(0, maxLength): validStr;
    final bytes = utf8.encode(subStr);
    
    return base64.encode(bytes);
  }

  String hash256({bool truncate = true}) {
    if (kDisableStringEncoding || this.isEmpty) {
      return this;
    }
    
    var bytes = utf8.encode(this);
    String hash = sha256.convert(bytes).toString();
    
    return truncate ? hash.substring(0, 8) : hash;
  }
}
