/*
 *  * Copyright © 2019 Pendo.io. All rights reserved.
 */

import 'dart:convert';

// TODO: enhance Logger
enum _Level {
  trace,
  debug,
  info,
  warning,
  error,
  critical,
  fatal
}

final Map<_Level, String> _defaultLevelEmojis = {
  _Level.trace: '',
  _Level.debug: '✅',
  _Level.info: 'ℹ️',
  _Level.warning: '⚠️',
  _Level.critical: '❗️',
  _Level.error: '⛔',
  _Level.fatal: '💀',
};

class Logger {
  /// sets the debug level print. use [_Level.trace] to print all messages/release.
  static _Level _debugLevel = _Level.error;
  
  static debugEnabled() {
    _debugLevel = _Level.trace;
  }

  static void t(String msg, {var object = null}) {
    _log(msg, object, _Level.trace);
  }

  static void d(String msg, {var object = null}) {
    _log(msg, object, _Level.debug);
  }

  static void i(String msg, {var object = null}) {
    _log(msg, object, _Level.info);
  }

  static void w(String msg, {var object = null}) {
    _log(msg, object, _Level.warning);
  }

  static void c(String msg, {var object = null}) {
    _log(msg, object, _Level.critical);
  }

  /// TODO: We need to consider option to send some errors we interested in, to the BE
  static void e(String msg, {var object = null}) {
    _log(msg, object, _Level.error);
  }

  static void f(String msg, {var object = null}) {
    _log(msg, object, _Level.fatal);
  }

  /// Private
  static _log(String msg, var obj, _Level lvl) {
    String prettyString = '';
    if (obj != null) {
      try {
        prettyString = JsonEncoder.withIndent('  ').convert(obj);
      } catch (e) {
        prettyString = '';
        print('could\'t prettify object. exception: $e');
      }
    }

    var constructedMsg = '$msg $prettyString';

    if (lvl.index >= _debugLevel.index) {
      String emoji = _defaultLevelEmojis[lvl] ?? '';
      String output = '[Pendo] ' + '[' + _getTime() + '] ' + emoji + ' '+ constructedMsg;
      _printWrapped(output);
    }
  }

  //https://stackoverflow.com/a/57579005/1790912
  static void _printWrapped(String text) {
    final pattern = RegExp('.{1,800}');
    pattern.allMatches(text).forEach((match) => print(match.group(0)));
  }

  static String _getTime({DateTime? time}) {
    if (time == null) {
      time = DateTime.now();
    }

    String threeDigits(int n) {
      if (n >= 100) return '$n';
      if (n >= 10) return '0$n';
      return '00$n';
    }

    String twoDigits(int n) {
      if (n >= 10) return '$n';
      return '0$n';
    }

    var now = time;
    var h = twoDigits(now.hour);
    var min = twoDigits(now.minute);
    var sec = twoDigits(now.second);
    var ms = threeDigits(now.millisecond);
    return '$h:$min:$sec.$ms';
  }
}