/*
 *  * Copyright © 2019 Pendo.io. All rights reserved.
 */
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import '../extensions/string_extension.dart';
// !! we ignore using relative imports only for known issue with logger.dart static filds
// ignore: directives_ordering, prefer_relative_imports
import 'package:pendo_sdk/src/logger.dart';

mixin ScreenIdParser {
  String? pageKeyExtractionFromRoute(Route route) {
    String? key;
    Page? page;

    if (route.settings is Page) {
      page = route.settings as Page;
      key = page.key?.toString().extractName();
      Logger.d('Extracting route name from the key: $key');
    }
    return key;
  }

  String extractRouteNameFromRouteTypes(Route route) {
    String routeName = _builderTypeDescriptionFromRoute(route);
    Logger.d('Extracting route name from route types: $routeName');
    return routeName;
  }

  String _builderTypeDescriptionFromRoute(Route route) {
    String routeRunType = route.runtimeType.toString();
    
    if (route is MaterialPageRoute) {
      String builderDescription = route.builder.toString();
      return routeRunType + builderDescription.extractReturnTypeFromBuilderDescription();
    }

    if (route is CupertinoPageRoute) {
      String builderDescription = route.builder.toString();
      return routeRunType + builderDescription.extractReturnTypeFromBuilderDescription();
    }

    if (route is ModalBottomSheetRoute) {
      String builderDescription = route.builder.toString();
      return routeRunType + builderDescription.extractReturnTypeFromBuilderDescription();
    }

    if (route is CupertinoModalPopupRoute) {
      String builderDescription = route.builder.toString();
      return routeRunType + builderDescription.extractReturnTypeFromBuilderDescription();
    }

    if (route is PageRouteBuilder) {
      String builderDescription = route.pageBuilder.toString();
      return routeRunType + builderDescription.extractReturnTypeFromBuilderDescription();
    }

    return routeRunType;
  }
}
