// ignore: prefer_relative_imports
import 'package:flutter/foundation.dart';
import '../constants/constants.dart';
import '../extensions/map_extension.dart';

class ScreenElementInfo {
  String? _selectedTitleText;
  String? get selectedTitleText => _selectedTitleText;

  int? _selectedIndex;
  int? get selectedIndex => _selectedIndex;

  int? _numberOfIndexes;
  int? get numberOfIndexes => _numberOfIndexes;

  String? _kind;
  String? get kind => _kind;

  int? _depth;
  int? get depth => _depth;

  String? _direction;
  String? get direction => _direction;

  int? _childrenCount;
  int? get childrenCount => _childrenCount;

  double? _itemExtent;
  double? get itemExtent => _itemExtent;

  double? _interval;
  double? get interval => _interval;

  int? _devision;
  int? get devision => _devision;

  int? _subDevision;
  int? get subDevision => _subDevision;

  int? _rows;
  int? get rows => _rows;

  int? _columns;
  int? get columns => _columns;

  ScreenElementInfo({
    String? selectedTitleText,
    int? selectedIndex,
    int? numberOfIndexes,
    String? kind,
    int? depth,
    String? direction,
    int? childrenCount,
    double? itemExtent,
    double? interval,
    int? devision,
    int? subDevision,
    int? rows,
    int? columns
  }):_selectedTitleText = selectedTitleText,
        _selectedIndex = selectedIndex,
        _numberOfIndexes = numberOfIndexes,
        _kind = kind,
        _depth = depth,
        _direction = direction,
        _childrenCount = childrenCount,
        _itemExtent = itemExtent,
        _interval = interval,
        _devision = devision,
        _subDevision = subDevision,
        _rows = rows,
        _columns = columns;

  ScreenElementInfo.copy(ScreenElementInfo info) {
    _selectedTitleText = info.selectedTitleText;
    _selectedIndex = info.selectedIndex;
    _numberOfIndexes = info.numberOfIndexes;
    _kind = info.kind;
    _depth = info.depth;
    _direction = info.direction;
    _childrenCount = info.childrenCount;
    _itemExtent = info.itemExtent;
    _interval = info.interval;
    _devision = info.devision;
    _subDevision = info.subDevision;
    _columns = info.columns;
    _rows = info.rows;
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    if (other is! ScreenElementInfo) return false;

    return runtimeType == other.runtimeType &&
      selectedTitleText == other.selectedTitleText &&
      numberOfIndexes == other.numberOfIndexes &&
      kind == other.kind &&
      depth == other.depth &&
      selectedIndex == other.selectedIndex &&
      direction == other.direction &&
      childrenCount == other.childrenCount &&
      itemExtent == other.itemExtent &&
      interval == other.interval &&
      devision == other.devision &&
      subDevision == other.subDevision &&
      rows == other.rows &&
      columns == other.columns;
  }

  @override
  int get hashCode => Object.hash(_selectedTitleText, _selectedIndex, _numberOfIndexes, _kind, _depth, _direction, _childrenCount, _itemExtent, _interval, _devision, _subDevision, _rows, _columns);

  Map<String, dynamic> toJson() {
    Map<String, dynamic> map = {};
    map.addIfNotNullOrEmpty(SELECTED_TITLE_TEXT, selectedTitleText);
    map.addIfNotNullOrEmpty(SELECTED_INDEX, selectedIndex);
    map.addIfNotNullOrEmpty(NUMBER_OF_INDEXES, numberOfIndexes);
    map.addIfNotNullOrEmpty(KIND, kind);
    map.addIfNotNullOrEmpty(DEPTH, depth);
    map.addIfNotNullOrEmpty(DIRECTION, direction);
    map.addIfNotNullOrEmpty(CHILDRENCOUNT, childrenCount);
    map.addIfNotNullOrEmpty(ITEMEXTENT, itemExtent);
    map.addIfNotNullOrEmpty(INTERVAL, interval);
    map.addIfNotNullOrEmpty(DEVISION, devision);
    map.addIfNotNullOrEmpty(SUBDEVISION, subDevision);
    map.addIfNotNullOrEmpty(ROWS, rows);
    map.addIfNotNullOrEmpty(COLUMNS, columns);
    return map;
  }
}

class ScreenChildElement {
  String _name;
  String get name => _name;

  int _depth;
  int get depth => _depth;

  ScreenElementInfo _info;
  ScreenElementInfo get info => _info;

  ScreenChildElement({required String name, required int depth, required ScreenElementInfo info}) :
        _name = name, _depth = depth, _info = info;

  ScreenChildElement.copy(ScreenChildElement viewPage): _name = viewPage.name, _depth = viewPage.depth, _info = ScreenElementInfo.copy(viewPage.info);

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    if (other is! ScreenChildElement) return false;

    return runtimeType == other.runtimeType &&
        _name == other._name &&
        _depth == other._depth &&
        _info == other._info; // Assuming ChildInfo also overrides ==
  }

  @override
  int get hashCode => Object.hash(_name, _depth, _info);

  Map<String, dynamic> toJson() {
    Map<String, dynamic> map = {};
    map[NAME] = name;
    map[DEPTH] = depth;
    map[INFO] = info.toJson();
    return map;
  }
}

class RetroactiveScreenData {
  String _retroactiveScreenId;
  String get retroactiveScreenId => _retroactiveScreenId;
  void set retroactiveScreenId(String screenId) {
    _retroactiveScreenId = screenId;
  }

  String _currentScreenName;
  String get currentScreenName => _currentScreenName;
  
  String _previousScreenName;
  String get previousScreenName => _previousScreenName;

  ScreenElementInfo? info;

  List<ScreenChildElement> _childViews = [];
  List<ScreenChildElement> get childViews => _childViews;

  Set<String> texts = {};

  RetroactiveScreenData({required String currentScreenName, required String previousScreenName, required String retroactiveScreenId }) :
    _currentScreenName = currentScreenName,
    _previousScreenName = previousScreenName,
    _retroactiveScreenId = retroactiveScreenId;

  RetroactiveScreenData.empty(): _currentScreenName = '', _previousScreenName = '', _retroactiveScreenId = '';

  RetroactiveScreenData.copy(RetroactiveScreenData otherData): 
    _currentScreenName = otherData.currentScreenName,
    _previousScreenName = otherData.previousScreenName,
    _retroactiveScreenId = otherData.retroactiveScreenId {
    if (otherData.info != null) {
      info = ScreenElementInfo.copy(otherData.info!);
    }
    for (ScreenChildElement child in otherData.childViews) {
      childViews.add(ScreenChildElement.copy(child));
    }
    texts = Set.from(otherData.texts);
  }

  Map<String, dynamic> toJson() {
    Map<String, dynamic> map = {};

    map[NAME] = currentScreenName;
    map[PREVIOUS_SCREEN_NAME] = previousScreenName;
    map[RETROACTIVE_SCREEN_ID] = retroactiveScreenId;
    map.addIfNotNullOrEmpty(INFO, info?.toJson());
    map.addIfNotNullOrEmpty(TEXTS, texts.toList());

    List<Map<String, dynamic>> children = [];
    for (ScreenChildElement child in _childViews) {
      children.add(child.toJson());
    }

    if (children.isNotEmpty) {
      map[INFO] = {
        CHILD_VIEWS: children,
        };
    }

    return map;
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    if (other is! RetroactiveScreenData) return false;

    RetroactiveScreenData otherData = other;
    return runtimeType == otherData.runtimeType &&
      _currentScreenName == otherData.currentScreenName &&
      _previousScreenName == otherData.previousScreenName &&
      _retroactiveScreenId == otherData.retroactiveScreenId &&
      info == otherData.info && // Assuming ChildInfo also overrides ==
      listEquals(_childViews, otherData.childViews) &&
      setEquals(texts, otherData.texts);
  }

  @override
  int get hashCode => Object.hash(_currentScreenName, _previousScreenName, _retroactiveScreenId, info);
}