import 'dart:ui';

import '../extensions/rect_extension.dart';

typedef TreeElementVisitor<V> = void Function(ITreeNode element);

abstract class ITree<T extends ITreeNode> {
  addTreeElement(T treeElement);
  Map<String, dynamic> toJson();
}

abstract class ITreeNode<T extends ITreeElementInfo> {
  T get elementInfo;
  int get elementDepth;

  addChild(ITreeNode<T> treeElement);
  bool isNestedClickableElement(ITreeNode<T> parentTreeElement) => elementInfo.isClickable && parentTreeElement.elementInfo.isClickable && parentTreeElement.elementInfo.frame.aproximateContainsRectangle(elementInfo.frame);
  bool shouldIgnoreElement(ITreeNode<T> otherTreeElement) => elementInfo.isClickable && otherTreeElement.elementInfo.isClickable; 
  bool shouldSquashChild(ITreeNode<T> parentTreeElement) => !elementInfo.isClickable && parentTreeElement.elementInfo.isClickable;
  
  String toString({int tab = 0});
  Map<String, dynamic> toJson();
}

abstract class ITreeElementInfo<T extends IWidgetInfo> {
  T get widgetInfo;
  bool get isClickable;
  int get elementDepth;
  Rect get frame;

  String toString({String indent});
  Map<String, dynamic> toJson();
}

abstract class IWidgetInfo {
  Map<String, dynamic> toJson();
}