import '../constants/constants.dart';
import 'element_info.dart';
import 'tree_abstract.dart';

class TreeNode extends ITreeNode<ElementInfo> {
  ElementInfo _elementInfo;
  ElementInfo get elementInfo => _elementInfo;

  List<ITreeNode<ElementInfo>> _children = [];
  List<ITreeNode<ElementInfo>> get children => _children;

  @override
  int get elementDepth => _elementInfo.elementDepth;

  TreeNode({ required ElementInfo elementInfo }) : _elementInfo = elementInfo;

  @override
  addChild(ITreeNode<ElementInfo> treeElement) {
    _children.add(treeElement);
  }

  @override
  String toString({int tab = 0}) {
    String result = '\t' * tab;
    result += _elementInfo.toString(indent: result);
    
    _children.forEach((element) {
      result += '${element.toString(tab: tab + 1)}  \n';
    });
    
    return result;
  }

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> map = {};
    map[ELEMENT_INFO] = elementInfo.toJson();
    map[CHILDREN] = [];
    
    _children.forEach((element) {
      (map[CHILDREN] as List).add(element.toJson());
    });
    
    return map;
  }
}