import 'package:flutter/material.dart';

// !! we ignore using relative imports only for known issue with logger.dart static filds
// ignore: prefer_relative_imports
import 'package:pendo_sdk/src/logger.dart';

import '../constants/constants.dart';
import '../extensions/clickables_extension.dart';
import '../extensions/iterable_extensions.dart';
import '../extensions/map_extension.dart';
import '../extensions/string_extension.dart';
import 'tree_abstract.dart';

class WidgetInfo extends IWidgetInfo {
  static const FILE_TAG = 'WidgetInfo';

  // General props
  String _widgetType;
  String get widgetType => _widgetType;

  String? _key;
  String? get key => _key;

  // Text props
  String? _text;
  String? get text => _text;

  String? _locale;
  String? get locale => _locale;

  String? _family;
  String? get family => _family;

  int? _color;
  int? get color => _color;

  bool? _enabled;
  bool? get enabled => _enabled;

  bool? _toggleable;
  bool? get toggleable => _toggleable;

  // Responsiveness 
  String? _onTap;
  String? get onTap => _onTap;

  String? _onTapUp;
  String? get onTapUp => _onTapUp;

  String? _onDoubleTap;
  String? get onDoubleTap => _onDoubleTap;

  String? _onPressed;
  String? get onPressed => _onPressed;

  String? _onLongPress;
  String? get onLongPress => _onLongPress;

  String? _onSelected;
  String? get onSelected => _onSelected;

  String? _onOpened;
  String? get onOpened => _onOpened;

  String? _onSelectionChanged;
  String? get onSelectionChanged => _onSelectionChanged;

  String? _onChanged;
  String? get onChanged => _onChanged; 

  String? _onDeleted;
  String? get onDeleted => _onDeleted; 

  String? _onSubmitted;
  String? get onSubmitted => _onSubmitted; 

  String? _onTapOutside;
  String? get onTapOutside => _onTapOutside; 

  String? _onEditingComplete;
  String? get onEditingComplete => _onEditingComplete;

  Object? _value;
  Object? get value => _value;

  String? _tooltipMessage;
  String? get tooltipMessage => _tooltipMessage;

  Object? _layoutId;
  Object? get layoutId => _layoutId;

  String? _semanticLabel;
  String? get semanticLabel => _semanticLabel;

  dynamic _label;
  dynamic get label => _label;

  dynamic _avatar;
  dynamic get avatar => _avatar;

  dynamic _deleteIcon;
  dynamic get deleteIcon => _deleteIcon;

  dynamic _icon;
  dynamic get icon => _icon;

  dynamic _selectedIcon;
  dynamic get selectedIcon => _selectedIcon;

  int? _iconCode;
  int? get iconCode => _iconCode;

  String? _imageName;
  String? get imageName => _imageName;
  
  int? _childCount;
  int? get childCount => _childCount;

  int? _index;
  int? get index => _index;

  double? _min;
  double? get min => _min;

  double? _max;
  double? get max => _max;

  bool? _tristate;
  bool? get tristate => _tristate;

  bool? _mini;
  bool? get mini => _mini;

  dynamic _child;
  dynamic get child => _child;

  dynamic _title;
  dynamic get title => _title;

  dynamic _subTitle;
  dynamic get subTitle => _subTitle;

  dynamic _leading;
  dynamic get leading => _leading;

  dynamic _trailing;
  dynamic get trailing => _trailing;

  bool? _isSelected;
  bool? get isSelected => _isSelected;

  bool _isClickable = false;
  bool get isClickable => _isClickable;

  bool _isContainer = false;
  bool get isContainer => _isContainer;

  int? _columns;
  int? get columns => _columns;

  int? _rows;
  int? get rows => _rows;

  WidgetInfo(Widget widget) : _widgetType = widget.runtimeType.toString() {
    _key = widget.key?.toString();
    // first we check if the widget is of type there is no additional info to look for
    if (widget.runtimeType.toString().contains('_Theater')) {
      return;
    } else if (widget.runtimeType.toString().contains('_OverlayEntryWidget')) {
      return;
    // then we handle the widgets that has useful additional info to collect
    // layout containers  
    } else if (widget is LayoutId) {
      getLayoutIdInfo();
    } else if (widget is Column) {
      getLaoyutContainerInfo();
    } else if (widget is Row) {
      getLaoyutContainerInfo();
    } else if (widget is Navigator) {
      getNavigatorInfo(widget);
    // text widgets 
    } else if (widget is Text) {
      getTextInfo(widget);
    } else if (widget is TextField) {
      getTextFieldInfo(widget);
    } else if (widget is RichText) {
      getRichTextInfo(widget);
    } else if (widget is SelectableText) {
      getSelectableTextInfo(widget);
    // image like widgets  
    } else if (widget is Image) {
      getImageInfo(widget);
    } else if (widget is Icon) {
      getIconInfo(widget);
    } else if (widget is Ink) {
      getInkInfo(widget);
    // responsive widgets
    } else if (widget is InkWell) {
      getInkWellInfo(widget);
    } else if (widget is InkResponse) {
      getInkResponseInfo(widget);
    } else if (widget is GestureDetector) {
      getGestureDetectorInfo(widget);
    } else if (widget is Checkbox) {
      getCheckboxInfo(widget);
    } else if (widget is Chip) {
      getChipInfo(widget);
    } else if (widget is ChoiceChip) {
      getChoiceChipInfo(widget);
    } else if (widget is FilterChip) {
      getFilterChipInfo(widget);
    } else if (widget is InputChip) {
      getInputChipInfo(widget);
    } else if (widget is ActionChip) {
      getActionChipInfo(widget);
    } else if (widget is Radio) {
      getRadioInfo(widget);
    } else if (widget is Switch) {
      getSwitchInfo(widget);
    } else if (widget is Slider) {
      getSliderInfo(widget);
    } else if (widget is TextButton) {
      getTextButtonInfo(widget);
    } else if (widget is FloatingActionButton) {
      getFloatingActionButtonInfo(widget);
    } else if (widget is IconButton) {
      getIconButtonInfo(widget);
    } else if (widget is ElevatedButton) {
      getElevatedButtonInfo(widget);
    } else if (widget is PopupMenuButton) {
      getPopupMenuButtonInfo(widget);
    }  else if (widget is GridView) {
      getGridViewInfo(widget);
    } else if (widget is ListView) {
      getListViewInfo(widget);
    } else if (widget is IndexedSemantics) {
      getIndexedSemanticsInfo(widget);
    } else if (widget is BottomNavigationBar) {
      getBottomNavigationBarInfo(widget);
    } else if (widget is TabBar) {
      getTabBarInfo(widget);
    } else if (widget is ListTile) {
      getListTileInfo(widget);
    } else if (widget is Tooltip) {
      getTooltipInfo(widget);
    } else if (widget is TabBarView) {
      getTabBarViewInfo(widget);
    } else if (widget is SliverFixedExtentList) {
      getSliverFixedExtentListInfo(widget);
    } else if (widget is GridPaper) {
      getGridPaperInfo(widget);
    } else if (widget is Flow) {
      getFlowInfo(widget);
    } else if (widget is DataTable) {
      getDataTableInfo(widget);
    } else {
      Logger.i('$FILE_TAG not implemented info extraction for: ${widget.runtimeType}');
    }
  }

  List<String> actions() {
    List<String>result = [];

    if (onTap != null) {
      result.addIfNotNull(onTap!.base64Str());
    }
    if (onTapUp != null) {
      result.addIfNotNull(onTapUp!.base64Str());
    }
    if (onDoubleTap != null) {
      result.addIfNotNull(onDoubleTap!.base64Str());
    }
    if (onPressed != null) {
      result.addIfNotNull(onPressed!.base64Str());
    }
    if (onLongPress != null) {
      result.addIfNotNull(onLongPress!.base64Str());
    }
    if (onSelected != null) {
      result.addIfNotNull(onSelected!.base64Str());
    }
    if (onOpened != null) {
      result.addIfNotNull(onOpened!.base64Str());
    }
    if (onSelectionChanged != null) {
      result.addIfNotNull(onSelectionChanged!.base64Str());
    }
    if (onChanged != null) {
      result.addIfNotNull(onChanged!.base64Str());
    }
    if (onDeleted != null) {
      result.addIfNotNull(onDeleted!.base64Str());
    }
    if (onSubmitted != null) {
      result.addIfNotNull(onSubmitted!.base64Str());
    }
    if (onTapOutside != null) {
      result.addIfNotNull(onTapOutside!.base64Str());
    }
    if (onEditingComplete != null) {
      result.addIfNotNull(onEditingComplete!.base64Str());
    }

    return result;
  }

  // Containers 
  void getLayoutIdInfo() {
    _layoutId = _key;
    _isContainer = true;
  }

  void getWrapInfo(Wrap widget) {
    _isContainer = true;
  }

  void getNavigatorInfo(Navigator widget) {
    _layoutId = widget.restorationScopeId;
    _isContainer = true;
  }

  void getLaoyutContainerInfo() {
    _isContainer = true;
  }

  void getListViewInfo(ListView widget) {
    _childCount = widget.semanticChildCount;
    _isContainer = true;
  }

  void getGridViewInfo(GridView widget) {
    _childCount = widget.semanticChildCount;
    _isContainer = true;
  }

  void getIndexedSemanticsInfo(IndexedSemantics widget) {
    _index = widget.index;
    _isContainer = true;
  }

  // Text elements
  void getTextInfo(Text widget) {
    _text = widget.data!.validateText();
    _locale = widget.locale?.toLanguageTag();
    _semanticLabel = widget.semanticsLabel;
    _color = widget.style?.color?.value;
    _family = widget.style?.fontFamily;
  }
  
  // TODO: We might want to skip rich text when it cames right after Text, Icon widget
  // TODO: There is another span's like WidgetSpan we need to investigate
  void getRichTextInfo(RichText widget) {
    InlineSpan inlineSpan = widget.text;
    if (inlineSpan is TextSpan) {
      _text = inlineSpan.toPlainText().validateText();
    } else if (inlineSpan is WidgetSpan) {
      _child = WidgetInfo(inlineSpan.child);
    } else {
      Logger.i('$FILE_TAG getRichTextInfo ${inlineSpan.runtimeType} not supported');
    }
  }

  void getTextFieldInfo(TextField widget) {
    _enabled = widget.enabled;

    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    if (widget.onChanged != null) {
      _onChanged = widget.onChanged.toString();
    }
    if (widget.onSubmitted != null) {
      _onSubmitted = widget.onSubmitted.toString();
    }
    if (widget.onTapOutside != null) {
      _onTapOutside = widget.onTapOutside.toString();
    }
    if (widget.onEditingComplete != null) {
      _onEditingComplete = widget.onEditingComplete.toString();
    }

    _isClickable = widget.isCallbackClickExists();
  }

  // There is might be SelectableText Widget with empty text we would want to skip
  void getSelectableTextInfo(SelectableText widget) {
    _text = widget.data!.validateText();

    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    if (widget.onSelectionChanged != null) {
      _onSelectionChanged = widget.onSelectionChanged.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  // There is might be two Image Widgets one after another, one with image and the next one
  // without that we would want to skip it
  void getImageInfo(Image widget) {
    _imageName = _imageInfoFromImageProvider(widget.image);
    _semanticLabel = widget.semanticLabel;
  }

  void getIconInfo(Icon widget) {
    _iconCode = widget.icon?.codePoint;
    _semanticLabel = widget.semanticLabel;
    _color = widget.color?.value;
  }

  void getAnimatedIconInfo(AnimatedIcon widget) {
    _semanticLabel = widget.semanticLabel;
  }

  // This is somtimes used as image however we might want to skip this element if there is no decoration
  void getInkInfo(Ink widget) {
    // There might be others '...Decoration' properties that might be interesting
    if (widget.decoration is BoxDecoration) {
      _imageName = _imageInfoFromImageProvider((widget.decoration! as BoxDecoration).image?.image); 
    } else if (widget.decoration is FlutterLogoDecoration) {
      _imageName = 'FlutterLogo';
    } else {
       Logger.i('$FILE_TAG getInkInfo ${widget.decoration?.runtimeType} not supported');
    }
  }

  // Clickable elements
  void getGestureDetectorInfo(GestureDetector widget) {
    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    if (widget.onDoubleTap != null) {
      _onDoubleTap = widget.onDoubleTap.toString();
    }
    if (widget.onLongPress != null) {
      _onLongPress = widget.onLongPress.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getInkResponseInfo(InkResponse widget) {
    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    if (widget.onDoubleTap != null) {
      _onDoubleTap = widget.onDoubleTap.toString();
    }
    if (widget.onLongPress != null) {
      _onLongPress = widget.onLongPress.toString();
    }
    if (widget.onTapUp != null) {
      _onTapUp = widget.onTapUp.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getCheckboxInfo(Checkbox widget) {
    _value = widget.value;
    _semanticLabel = widget.semanticLabel;
    _tristate = widget.tristate;
    if (widget.onChanged != null) {
      _onChanged = widget.onChanged.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getChipInfo(Chip widget) {
    _label = WidgetInfo(widget.label);
    if (widget.avatar != null) {
      _avatar = WidgetInfo(widget.avatar!);   
    }
    if (widget.deleteIcon != null) {
      _deleteIcon = WidgetInfo(widget.deleteIcon!);
    }
    if (widget.onDeleted != null) {
      _onDeleted = widget.onDeleted.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getChoiceChipInfo(ChoiceChip widget) {
    _label = WidgetInfo(widget.label);
    if (widget.avatar != null) {
      _avatar = WidgetInfo(widget.avatar!);   
    }
    _enabled = widget.isEnabled;
    _isSelected = widget.selected;
    _tooltipMessage = widget.tooltip;
    if (widget.onSelected != null) {
      _onSelected = widget.onSelected.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getFilterChipInfo(FilterChip widget) {
    _label = WidgetInfo(widget.label);
    if (widget.avatar != null) {
      _avatar = WidgetInfo(widget.avatar!);   
    }
    _enabled = widget.isEnabled;
    _isSelected = widget.selected;
    _tooltipMessage = widget.tooltip;
    if (widget.onSelected != null) {
      _onSelected = widget.onSelected.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getInputChipInfo(InputChip widget) {
    _label = WidgetInfo(widget.label);
    if (widget.avatar != null) {
      _avatar = WidgetInfo(widget.avatar!);   
    }
    if (widget.deleteIcon != null) {
      _deleteIcon = WidgetInfo(widget.deleteIcon!);
    }
    _enabled = widget.isEnabled;
    _isSelected = widget.selected;
    _tooltipMessage = widget.tooltip;
    if (widget.onPressed != null) {
      _onPressed = widget.onPressed.toString();
    }
    if (widget.onDeleted != null) {
      _onDeleted = widget.onDeleted.toString();
    }
    if (widget.onSelected != null) {
      _onSelected = widget.onSelected.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

   void getActionChipInfo(ActionChip widget) {
    _label = WidgetInfo(widget.label);
    if (widget.avatar != null) {
      _avatar = WidgetInfo(widget.avatar!);   
    }
    _enabled = widget.isEnabled;
    _tooltipMessage = widget.tooltip;
    if (widget.onPressed != null) {
      _onPressed = widget.onPressed.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getSwitchInfo(Switch widget) {
    _value = widget.value;
    if (widget.onChanged != null) {
      _onChanged = widget.onChanged.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getRadioInfo(Radio widget) {
    _color = widget.activeColor?.value;
    _value = widget.value;
    _toggleable = widget.toggleable;
    _onChanged = widget.onChanged?.toString();
    _isClickable = widget.isCallbackClickExists();
  }

  void getSliderInfo(Slider widget) {
    _text = widget.label!.validateText();
    _value = widget.value;
    _min = widget.min;
    _max = widget.max;
    if (widget.onChanged != null) {
      _onChanged = widget.onChanged.toString();
    }
  }

  void getPopupMenuButtonInfo(PopupMenuButton widget) {
    if (widget.icon != null) {
      _icon = WidgetInfo(widget.icon!);
    }
    _color = widget.color?.value;
    _enabled = widget.enabled;
    _tooltipMessage = widget.tooltip;
    _isClickable = true;
  }

  void getInkWellInfo(InkWell widget) {
    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    if (widget.onDoubleTap != null) {
      _onDoubleTap = widget.onDoubleTap.toString();
    }
    if (widget.onLongPress != null) {
      _onLongPress = widget.onLongPress.toString();
    }
    if (widget.onTapUp != null) {
      _onTapUp = widget.onTapUp.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getIconButtonInfo(IconButton widget) {
    _color = widget.color?.value;
    _icon = WidgetInfo(widget.icon);
    if (widget.selectedIcon != null) {
      _selectedIcon = WidgetInfo(widget.selectedIcon!);
    }
    _isSelected = widget.isSelected;
    _tooltipMessage = widget.tooltip;
    if (widget.onPressed != null) {
      _onPressed = widget.onPressed.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getTextButtonInfo(TextButton widget) {
    if (widget.child != null) {
      _child = WidgetInfo(widget.child!);
    }
    _enabled = widget.enabled;
    if (widget.onPressed != null) {
      _onPressed = widget.onPressed.toString();
    }
    if (widget.onLongPress != null) {
      _onLongPress = widget.onLongPress.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getElevatedButtonInfo(ElevatedButton widget) {
    if (widget.child != null) {
      _child = WidgetInfo(widget.child!);
    }
    _enabled = widget.enabled;
    if (widget.onPressed != null) {
      _onPressed = widget.onPressed.toString();
    }
    if (widget.onLongPress != null) {
      _onLongPress = widget.onLongPress.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getFloatingActionButtonInfo(FloatingActionButton widget) {
    if (widget.child != null) {
      _child = WidgetInfo(widget.child!);
    }
    _mini = widget.mini;
    _tooltipMessage = widget.tooltip;
    if (widget.onPressed != null) {
      _onPressed = widget.onPressed.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getListTileInfo(ListTile widget) {
    if (widget.title != null) {
      _title = WidgetInfo(widget.title!);
    }
    if (widget.subtitle != null) {
      _subTitle = WidgetInfo(widget.subtitle!);
    }
    if (widget.leading != null) {
      _leading = WidgetInfo(widget.leading!);
    }
    if (widget.trailing != null) {
      _trailing = WidgetInfo(widget.trailing!);
    }
    _enabled = widget.enabled;
    _isSelected = widget.selected;
    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    if (widget.onLongPress != null) {
      _onLongPress = widget.onLongPress.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getTabBarInfo(TabBar widget) {
    _childCount = widget.tabs.length;
    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getBottomNavigationBarInfo(BottomNavigationBar widget) {
    _childCount = widget.items.length;
    if (widget.onTap != null) {
      _onTap = widget.onTap.toString();
    }
    _isClickable = widget.isCallbackClickExists();
  }

  void getTooltipInfo(Tooltip widget) {
    _tooltipMessage = widget.message;
  }

  void getTabBarViewInfo(TabBarView widget) {
    _isContainer = true;
    var controller = widget.controller;

    if (controller != null && controller.length > 0) {
      _childCount = controller.length;
    }
  }
    
  void getSliverFixedExtentListInfo(SliverFixedExtentList widget) {
    _isContainer = true;
  }

  void getGridPaperInfo(GridPaper widget) {
    _isContainer = true;
    _color = widget.color.value;
  }

  void getFlowInfo(Flow widget) {
    _isContainer = true;
    _childCount = widget.children.length;
  }

  void getDataTableInfo(DataTable widget) {
    _isContainer = true;
    _columns = widget.columns.length;
    _rows = widget.rows.length;
  }

  String? _imageInfoFromImageProvider(ImageProvider? imageProvider) {
    if (imageProvider == null) {
      return null;
    }

    if (imageProvider is AssetImage) {
      return imageProvider.assetName;
    }

    if (imageProvider is ExactAssetImage) {
      return imageProvider.assetName;
    }

    if (imageProvider is NetworkImage) {
      return imageProvider.url;
    }

    if (imageProvider is FileImage) {
      return imageProvider.file.path;
    }

    if (imageProvider is ResizeImage) {
      return _imageInfoFromImageProvider(imageProvider.imageProvider);
    }

    if (imageProvider is ScrollAwareImageProvider) {
      return _imageInfoFromImageProvider(imageProvider.imageProvider);
    }

    Logger.w('Image extraction failed for image provider: ${imageProvider.runtimeType}');
    return null;
  }

  @visibleForTesting
  List<String> getTexts() {
    final List<String> result = [];

    if (text != null) {
      result.add(text!);
    }
    if (title != null && title is WidgetInfo) {
      result.addAll((title as WidgetInfo).getTexts());
    }
    if (subTitle != null && subTitle is WidgetInfo) {
      result.addAll((subTitle as WidgetInfo).getTexts());
    }

    if (child != null && child is WidgetInfo) {
      result.addAll((child as WidgetInfo).getTexts());
    }

    return result;
  }

  String? getText() {
    final List<String> texts = getTexts();
    return texts.isNotEmpty ? texts.first.base64Str() : null;
  }

  void squashChild(WidgetInfo child) {
    if (child.text != null && text != child.text) {
      if (text == null) {
        _text = child.text;
      } else if (!text!.contains(child.text!)) {
        _text = '$text\_${child.text}';
      }
    }

    if (child.imageName != null && imageName != child.imageName) {
      if (imageName == null) {
        _imageName = child.imageName;
      } else if (!imageName!.contains(child.imageName!)) {
        _imageName = '$imageName\_${child.imageName}';
      }
    }
    
    if (child.iconCode != null && iconCode != child.iconCode) {
      if (iconCode == null) {
        _iconCode = child.iconCode;
      } else if (iconCode != child.iconCode!) {
        var codeStr = '$iconCode${child.iconCode}';
        _iconCode = int.parse(codeStr);
      }
    }
  }

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> map = {};

    map[WIDGET_TYPE] = _widgetType;
    map[IS_CONTAINER] = isContainer;
    map[IS_CLICKABLE] = isClickable;

    map.addIfNotNullOrEmpty(KEY, key);
    map.addIfNotNullOrEmpty(TEXT, text);
    map.addIfNotNullOrEmpty(LOCALE, locale);
    map.addIfNotNullOrEmpty(FONT_FAMILY, family);
    map.addIfNotNullOrEmpty(COLOR, color);
    map.addIfNotNullOrEmpty(ENABLED, enabled);
    map.addIfNotNullOrEmpty(TOGGLEABLE, toggleable);
    map.addIfNotNullOrEmpty(ON_TAP, onTap);
    map.addIfNotNullOrEmpty(ON_TAP_UP, onTapUp);
    map.addIfNotNullOrEmpty(ON_DOUBLE_TAP, onDoubleTap);
    map.addIfNotNullOrEmpty(ON_PRESSED, onPressed);
    map.addIfNotNullOrEmpty(ON_LONG_PRESS, onLongPress);
    map.addIfNotNullOrEmpty(ON_SELECTED, onSelected);
    map.addIfNotNullOrEmpty(ON_OPENED, onOpened);
    map.addIfNotNullOrEmpty(ON_SELECTION_CHANGED, onSelectionChanged);
    map.addIfNotNullOrEmpty(ON_CHANGED, onChanged);
    map.addIfNotNullOrEmpty(ON_DELETED, onDeleted);
    map.addIfNotNullOrEmpty(ON_SUBMITTED, onSubmitted);
    map.addIfNotNullOrEmpty(ON_TAP_OUTSIDE, onTapOutside);
    map.addIfNotNullOrEmpty(ON_EDITTING_COMPLETE, onEditingComplete);
    map.addIfNotNullOrEmpty(VALUE, value);
    map.addIfNotNullOrEmpty(TOOLTIP_MESSAGE, tooltipMessage);
    map.addIfNotNullOrEmpty(LAYOUT_ID, layoutId);
    map.addIfNotNullOrEmpty(SEMANTIC_LABEL, semanticLabel);
    map.addIfNotNullOrEmpty(ON_EDITTING_COMPLETE, onEditingComplete);
    map.addIfNotNullOrEmpty(ICON_CODE, iconCode);
    map.addIfNotNullOrEmpty(IMAGE_NAME, imageName);
    map.addIfNotNullOrEmpty(CHILD_COUNT, childCount);
    map.addIfNotNullOrEmpty(INDEX, index);
    map.addIfNotNullOrEmpty(MIN, min);
    map.addIfNotNullOrEmpty(MAX, max);
    map.addIfNotNullOrEmpty(TRISTATE, tristate);
    map.addIfNotNullOrEmpty(MINI, mini);
    map.addIfNotNullOrEmpty(IS_SELECTED, isSelected);
    map.addIfNotNullOrEmpty(ROWS, rows);
    map.addIfNotNullOrEmpty(COLUMNS, columns);

    addWidgetInfoIfExist(map, LABEL, label);
    addWidgetInfoIfExist(map, AVATAR, avatar);
    addWidgetInfoIfExist(map, DELETE_ICON, deleteIcon);
    addWidgetInfoIfExist(map, ICON, icon);
    addWidgetInfoIfExist(map, SELECTED_ICON, selectedIcon);
    addWidgetInfoIfExist(map, CHILD, child);
    addWidgetInfoIfExist(map, TITLE, title);
    addWidgetInfoIfExist(map, SUBTITLE, subTitle);
    addWidgetInfoIfExist(map, LEADING, leading);
    addWidgetInfoIfExist(map, TRAILING, trailing);

    return map;
  }

  void addWidgetInfoIfExist(Map<String, dynamic> map, String key, value) {
    if (value != null) {
      if (value is WidgetInfo) {
        // ignore: unnecessary_cast
        map.addIfNotNullOrEmpty(key, (value as WidgetInfo).toJson());
      } else {
        map.addIfNotNullOrEmpty(key, value);
      }
    }
  }
}
